/*
** psmisk.c - A gentle process smisker
**
** Copyright (c) 2004 Peter Eriksson <pen@lysator.liu.se>
**
** This program is free software; you can redistribute it and/or
** modify it as you wish - as long as you don't claim that you wrote
** it. But only a fool would try to make that claim anyway.
**
** This program is distributed in the hope that it wont be useful,
** any WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include <sys/types.h>
#include <signal.h>
#include <time.h>


int verbose = 0;
int randsig = 0;


int
main(int argc,
     char *argv[])
{
    int i, j, rc, pid, sig;

    srandom(time(NULL));

    for (i = 1; i < argc && argv[i][0] == '-'; i++)
	for (j = 1; argv[i][j]; j++)
	    switch (argv[i][j])
	    {
	      case 'r':
		randsig = 1;
		break;
		
	      case 'v':
		++verbose;
		break;

	      case 'm':
		execl("/bin/mail", "mail", NULL);
		execl("/usr/bin/mail", "mail", NULL);
		fprintf(stderr, "%s: exec of 'mail' failed: %s\n",
			argv[0], strerror(errno));
		exit(1);
		
	      case 'h':
		printf("Usage: %s [options]  <pid-1> [... <pid-N>]\n",
		       argv[0]);
		puts("Options:");
		puts("\t-h\tDisplay this help text");
		puts("\t-v\tBe verbose");
		puts("\t-r\tUse random signal instead of WINCH");
		puts("\t-m\tRead mail.");
		exit(0);

 	      default:
		fprintf(stderr, "%s: invalid switch: -%s\n",
			argv[0], argv[i]+j);
		exit(1);
	    }
	    
    if (i == argc)
    {
	fprintf(stderr, "%s: missing arguments\n",
		argv[0]);
	exit(1);
    }

    for (; i < argc; i++)
    {
	if (sscanf(argv[i], "%u", &pid) != 1)
	{
	    fprintf(stderr, "%s: invalid argument number %u: %s\n",
		    argv[0], i, argv[i]);
	    exit(1);
	}

	if (verbose)
	{
	    printf("Smisking process %u...", pid);
	    fflush(stdout);
	}

	if (randsig)
	    while ((rc = kill(pid, (sig = random()%64))) < 0 && errno == EINVAL)
		;
	else
	    rc = kill(pid, (sig = SIGWINCH));

	if (rc < 0)
	{
	    if (verbose)
		printf(" Failed smisking with signal %u: %s\n",
		       sig, strerror(errno));
	    else
		fprintf(stderr, "%s: smisking of process %u with signal %u failed: %s\n",
		    argv[0], pid, sig, strerror(errno));
	}
	else
	    if (verbose)
		printf(" Smisked with signal %u.\n", sig);
    }

    return 0;
}
